/*  GWhere
 *  Copyright (C) 2000  Sbastien LECACHEUR
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "../gwsupport.h"
#include "gwsettingsexplorer.h"

#include "gwsettingswindowbox.h"
#include "gwsettingsexplorerdisk.h"
#include "gwsettingsexplorerfile.h"

#include "../gwapplicationmanager.h"


/*! @define	GW_PLUGIN_SETTINGS_WINDOW	The parent window */
#define GW_PLUGIN_SETTINGS_WINDOW "gw_plugin_settings_window"


gint gw_plugin_settings_explorer_pane_create ( GtkWindow *settings, GtkContainer *parent, GtkWidget **pane);
gint gw_plugin_settings_explorer_pane_load ( GtkWidget *pane);
gint gw_plugin_settings_explorer_pane_on_change ( GtkEntry *entry, GtkWidget *pane);
gint gw_plugin_settings_explorer_pane_apply ( GtkWidget *pane);


gint gw_plugin_settings_explorer_init ( GWSettingsModule **module)
{
	gint result = -1;

	if ( (*module = gw_settings_module_new ( )) != NULL )
	{
		(*module)->name = g_strdup ( _( "Explorer"));
		(*module)->create = &gw_plugin_settings_explorer_pane_create;
		(*module)->load = &gw_plugin_settings_explorer_pane_load;
		(*module)->on_change = &gw_plugin_settings_explorer_pane_on_change;
		(*module)->apply = &gw_plugin_settings_explorer_pane_apply;

		(*module)->child = (GWSettingsModule**)g_malloc ( sizeof ( GWSettingsModule*) * 3);

		if ( (*module)->child != NULL)
		{
			(*module)->child[0] = NULL;
			(*module)->child[1] = NULL;
			(*module)->child[2] = NULL;
			gw_plugin_settings_explorer_disk_init ( &((*module)->child[0]));
			gw_plugin_settings_explorer_file_init ( &((*module)->child[1]));
		}

		result = 0;
	}

	return result;
}


gint gw_plugin_settings_explorer_pane_create ( GtkWindow *settings, GtkContainer *parent, GtkWidget **pane)
{
	gint result = -1;
	GtkWidget *hbox;
	GtkTooltips *tooltips;


#ifdef GW_DEBUG_PLUGIN_SETTINGS_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	tooltips = gtk_tooltips_new ( );

	if ( settings != NULL && parent != NULL )
	{
		hbox = gtk_hbox_new ( FALSE, 0);
		gtk_widget_ref ( GTK_WIDGET ( settings));
		gtk_object_set_data_full ( GTK_OBJECT ( hbox), GW_PLUGIN_SETTINGS_WINDOW, settings, (GtkDestroyNotify) gtk_widget_unref);
		gtk_container_set_border_width ( GTK_CONTAINER ( hbox), 5);

		*pane = hbox;

		gw_plugin_settings_explorer_pane_load ( hbox);

		/* Connects signals after loading. */

		result = 0;
	}

	return result;
}


gint gw_plugin_settings_explorer_pane_load ( GtkWidget *pane)
{
	gint result = -1;


#ifdef GW_DEBUG_PLUGIN_SETTINGS_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( pane != NULL )
	{
		result = 0;
	}

	return result;
}


gint gw_plugin_settings_explorer_pane_on_change ( GtkEntry *entry, GtkWidget *pane)
{
	gint result = -1;
	GtkWindow *settings = NULL;


#ifdef GW_DEBUG_PLUGIN_SETTINGS_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( pane != NULL )
	{
		if ( (settings = GTK_WINDOW ( gtk_object_get_data ( GTK_OBJECT ( pane), GW_PLUGIN_SETTINGS_WINDOW))) != NULL )
		{
			gw_settings_window_box_set_modified ( settings, TRUE);

			result = 0;
		}
	}

	return result;
}


gint gw_plugin_settings_explorer_pane_apply ( GtkWidget *pane)
{
	gint result = -1;
	GtkWindow *settings = NULL;


#ifdef GW_DEBUG_PLUGIN_SETTINGS_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( pane != NULL )
	{
		if ( (settings = GTK_WINDOW ( gtk_object_get_data ( GTK_OBJECT ( pane), GW_PLUGIN_SETTINGS_WINDOW))) != NULL )
		{
			gw_settings_window_box_set_modified ( settings, FALSE);
		}

		result = 0;
	}

	return result;
}


GWSettingsExplorerField * gw_settings_explorer_field_new ( void)
{
	GWSettingsExplorerField *p = NULL;


#ifdef GW_DEBUG_PLUGIN_SETTINGS_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( (p = (GWSettingsExplorerField*)g_malloc ( sizeof ( GWSettingsExplorerField))) != NULL )
	{
		p->name = NULL;
		p->position = -1;
		p->position_v = NULL;
		p->visible = FALSE;
		p->visible_v = NULL;
	}

	return p;
}


GWSettingsExplorerField * gw_settings_explorer_field_find_by_name ( GWSettingsExplorerField **p, const gchar *name)
{
	GWSettingsExplorerField *field = NULL;
	gint i = 0;


#ifdef GW_DEBUG_PLUGIN_SETTINGS_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( (p != NULL) && (name != NULL) )
	{
		while ( p[i] != NULL )
		{
			if ( (p[i]->name != NULL) && (strcmp ( name, p[i]->name) == 0) )
			{
				field = p[i];
			}

			i++;
		}
	}

	return field;
}


gint gw_settings_explorer_field_free ( GWSettingsExplorerField * p)
{
	gint result = -1;


#ifdef GW_DEBUG_PLUGIN_SETTINGS_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( p != NULL )
	{
		if ( p->name != NULL )
		{
			g_free ( p->name);
			p->name = NULL;
		}

		if ( p->position_v != NULL )
		{
			g_free ( p->position_v);
			p->position_v = NULL;
		}

		if ( p->visible_v != NULL )
		{
			g_free ( p->visible_v);
			p->visible_v = NULL;
		}

		g_free ( p);

		result = 0;
	}

	return result;
}


gint gw_settings_explorer_field_freev ( GWSettingsExplorerField **p)
{
	gint result = -1;
	gint i = 0;


#ifdef GW_DEBUG_PLUGIN_SETTINGS_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( p != NULL )
	{
		for ( i = 0; p[i] != NULL; i++)
		{
			gw_settings_explorer_field_free ( p[i]);
			p[i] = NULL;
		}

		g_free ( p);
		result = 0;
	}

	return result;
}
