/*  GWhere.
 *  Copyright (C) 2000  Sbastien LECACHEUR
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "gwsupport.h"
#include "gwdevicemanager.h"


/* To open and close files */
#include <unistd.h> /* lseek read close */
#include <fcntl.h> /* open */

#include <stdlib.h> /* exit */
#include <errno.h> /* errno */
#include <string.h> /* strncmp malloc*/
#include <stdio.h> /* perror*/
#include <ctype.h> /* toupper */

/* To mount device */
#include <sys/stat.h> /* stat */

#if defined ( OS_LINUX) || defined ( OS_CYGWIN)
    #include <mntent.h> /* setmntent getmntent endmntent */
    #include <sys/mount.h> /* mount umount */
    #include <sys/wait.h>
#endif

/* To get information on the file system */
#if defined ( OS_LINUX) || defined ( OS_CYGWIN)
    #include <sys/vfs.h> /* statfs */
#endif

#if defined ( OS_WIN32) || defined ( OS_CYGWIN)
	#include <windows.h>
 	#include <winioctl.h>
#endif

#include "tools/gwtools.h"

#include "gwapplicationmanager.h"
#include "gwsupermount.h"


gint gw_devicemanager_device_load_to_vfs_stats ( struct vfs_stats *p, gchar *dir)
{
	/* Loads all file system  informations */
#if defined ( OS_CYGWIN) || defined ( OS_WIN32)
	gchar *device_name = NULL;
	gchar *volume_name = NULL;
	gchar *serial_number = NULL;
	gchar *device_type = NULL;
	gulong nb = 0;
#endif


#ifdef GW_DEBUG_MODE
	gw_application_manager_log_msg ( 0, __FILE__, __LINE__, __PRETTY_FUNCTION__, NULL);
#endif

	if ( p == NULL )
	{
		/* The virtual file system hasn't been allocated */
		g_print ( "p == (null)\n");

		return -1;
	}

	if ( dir == NULL )
	{
		/* The file system path is NULL */
		g_print ( "dir == (null)\n");

		return -1;
	}

#if defined ( OS_LINUX)
	/* Checks if the device is in the mounted device table (/etc/mtab) */
	if ( gw_device_manager_search_disk_in_tab_file_to_vfs_stats ( p, "/etc/mtab", dir) == 0 )
	{
		return 0;
	}

	/* Checks if the device is in the mountabled device table (/etc/fstab) */
	if ( gw_device_manager_search_disk_in_tab_file_to_vfs_stats ( p, "/etc/fstab", dir) == 0 )
	{
		return 0;
	}
#elif defined ( OS_CYGWIN_DEPRECATED)
	gw_device_manager_disk_get_device_name ( dir, &device_name);
	vfs_stats_set_name ( p , device_name);
	vfs_stats_set_dir ( p, g_strdup ( dir));
	vfs_stats_set_options ( p, NULL);
	gw_fs_get_type ( device_name, &device_type);
	vfs_stats_set_type ( p , device_type);
	vfs_stats_set_ismounted ( p, TRUE);
//	vfs_stats_set_nb_files ( p, 0);
//	vfs_stats_set_blocks_size ( p, 0);
//	vfs_stats_set_blocks ( p, 0);
//	vfs_stats_set_blocks_free ( p, 0);
//	vfs_stats_set_label ( p, vfs_stats_disk_get_label ( p));
	gw_fs_get_volume_name ( device_name, &volume_name);
	vfs_stats_set_label ( p, volume_name);
#elif defined ( OS_WIN32) || defined ( OS_CYGWIN)
	device_name = g_strdup_printf ( "\\\\.\\%s", dir);

	vfs_stats_set_name ( p , device_name);
	vfs_stats_set_dir ( p, g_strdup ( dir));
	vfs_stats_set_options ( p, NULL);
	gw_fs_get_type ( device_name, &device_type);
	if ( device_type == NULL )
	{
		switch ( GetDriveType ( device_name))
		{
			case DRIVE_UNKNOWN:
	
						break;
	
			case DRIVE_NO_ROOT_DIR:
	
						break;

			case DRIVE_REMOVABLE:	switch ( toupper ( dir[0]))
						{
							case 'A':	g_strdup ( "/dev/fd0");break;
							case 'B':	g_strdup ( "/dev/fd1");break;
						}
	
						break;
	
			case DRIVE_FIXED:
	
						break;
	
			case DRIVE_REMOTE:
	
						break;
	
			case DRIVE_CDROM:	device_type = g_strdup ( "iso9660");
	
						break;

			case DRIVE_RAMDISK:

						break;

			default : break;
		}
	}

	vfs_stats_set_type ( p , device_type);
	vfs_stats_set_ismounted ( p, TRUE);
//	vfs_stats_set_nb_files ( p, 0);
	vfs_stats_set_blocks_size ( p, 1024);
	nb = gw_drive_get_capacity ( device_name);
	vfs_stats_set_blocks ( p, nb);
	nb = gw_drive_get_free_space ( device_name);
	vfs_stats_set_blocks_free ( p, nb);
	gw_fs_get_serial_number ( device_name, &serial_number);
	vfs_stats_set_serial_number ( p, serial_number);
	gw_fs_get_volume_name ( device_name, &volume_name);
	vfs_stats_set_label ( p, volume_name);

	return 0;
#endif

	/* If the path file system doesn't exist */
	return -1;
}




gboolean gw_device_manager_disk_try_mount_from_vfs_stats ( struct vfs_stats *p)
{
/*
// First methode failed : single user cannot use mount()

	if ( mount ( vfs_stats_get_name ( p), vfs_stats_get_dir ( p), vfs_stats_get_type ( p), 0xc0ed0000|MS_RDONLY, "") == -1 )
	{
		if ( errno == EBUSY )
		{
g_print ( "try_mount : device busy\n");
			return TRUE;
		}
		else
		{
g_print ( "try_mount : cannot mount\n");
			return FALSE;
		}
	}
	else
	{
		umount ( vfs_stats_get_dir ( p));

g_print ( "try_mount : can mount\n");
		return TRUE;
	}
*/
/*
// Second methode failed : cannot detect if the device is allready mounted
	if ( gw_device_manager_disk_mount_from_vfs_stats ( p) == TRUE )
	{
		gw_device_manager_disk_umount_from_vfs_stats ( p);

		return TRUE;
	}

	return FALSE;
*/
/*
// Third methode failed : umount an umounted device doesn't failed
	if ( gw_device_manager_disk_umount_from_vfs_stats ( p) == TRUE )
	{
		gw_device_manager_disk_mount_from_vfs_stats ( p);

		return TRUE;
	}
	else
	{
		if ( gw_device_manager_disk_mount_from_vfs_stats ( p) == TRUE )
		{
			gw_device_manager_disk_umount_from_vfs_stats ( p);

			return TRUE;
		}

	return FALSE;
	}

	return FALSE;
*/
	/* Gets the label of a disk */
	struct stat statbuf;
	guint fic;


#ifdef GW_DEBUG_MODE
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	/* Gets rights of the disk */
	if ( stat ( vfs_stats_get_name ( p), &statbuf) == -1 )
	{
		perror ( "stat");

		return FALSE;
	}

	/* If it's a regular file */
	if ( S_ISREG ( statbuf.st_mode) )
	{
		return FALSE;
	}

	/* Cannot read only the file */
	if ( access ( vfs_stats_get_name ( p), R_OK) != 0 )
	{
        		return FALSE;
	}

	/* Opens the device */
	if ( ( fic = open ( vfs_stats_get_name ( p), O_RDONLY ) ) == -1 )
	{
/*		perror ( "open");*/

		return FALSE;
	}

	/* Closes the device file system */
	close ( fic);

	return TRUE;
}


gint gw_device_manager_search_disk_in_tab_file_to_vfs_stats ( struct vfs_stats *p, const gchar *file_tab, const gchar *dir)
{
	FILE *tab = NULL;
#if defined ( OS_LINUX) || defined ( OS_CYGWIN)
	struct mntent *partition = NULL;
	struct statfs sts;
	gchar *volume_name = NULL;
	gulong size = 0;
	gboolean has_mounted = FALSE;


#ifdef GW_DEBUG_MODE
	gw_application_manager_log_msg ( 0, __FILE__, __LINE__, __PRETTY_FUNCTION__, NULL);
#endif

	if ( p == NULL )
	{
		/* The virtual file system hasn't been allocated */
		return -1;
	}

	if ( dir == NULL )
	{
		/* The file system path is NULL */
		return -1;
	}

	tab = (FILE*)malloc ( sizeof ( tab));
	if ( (partition = (struct mntent*)malloc ( sizeof ( struct mntent))) == NULL)
	{
		return -1;
	}

	/* Opens the device table file */
	tab = setmntent ( file_tab, "r");

	while ( (partition = getmntent ( tab)) != NULL)
	{
             	if ( strcmp ( partition->mnt_dir, dir) == 0 )
		{
			/* Bug to fix : must free all data before set new informations */
			vfs_stats_set_name ( p , g_strdup ( partition->mnt_fsname));
			vfs_stats_set_dir ( p, g_strdup ( partition->mnt_dir));
			vfs_stats_set_options ( p, g_strsplit ( partition->mnt_opts, ",", 0));
			vfs_stats_set_type ( p , g_strdup ( partition->mnt_type));

			if ( strncmp ( file_tab, "/etc/mtab", 9) == 0 )
			{
				/* If the device table file is /etc/mtab */
				vfs_stats_set_ismounted ( p, TRUE);
			}
			else { if ( strncmp ( file_tab, "/etc/fstab", 10) == 0 )
			{
				/* If the device table file is /etc/fstab */
				vfs_stats_set_ismounted ( p, FALSE);
			}}

			if ( gw_device_manager_disk_can_mounted_from_vfs_stats ( p) )
			{

				if ( gw_device_manager_disk_is_mounted_from_vfs_stats ( p) )
				{
					if ( (vfs_stats_get_type ( p) != NULL) && (strcmp ( vfs_stats_get_type ( p), "iso9660") == 0) )
					{
						volume_name = gw_fs_iso9660_get_volume_name ( vfs_stats_get_name ( p));
					}
					else
					{
						gw_fs_get_volume_name ( vfs_stats_get_name ( p), &volume_name);
					}
					vfs_stats_set_label ( p, volume_name);
				}
				else
				{
					has_mounted = gw_device_manager_disk_mount_from_vfs_stats ( p);

					if ( (vfs_stats_get_type ( p) != NULL) && (strcmp ( vfs_stats_get_type ( p), "iso9660") == 0) )
					{
						volume_name = gw_fs_iso9660_get_volume_name ( vfs_stats_get_name ( p));
					}
					else
					{
						gw_fs_get_volume_name ( vfs_stats_get_name ( p), &volume_name);
					}
					vfs_stats_set_label ( p, volume_name);

					if ( !vfs_stats_get_ismounted ( p) && has_mounted )
					{
						gw_device_manager_disk_umount_from_vfs_stats ( p);
					}
				}

			if ( statfs ( dir, &sts) == -1 )
			{
				perror ( "statfs");

				return -1;
			}

			size = sts.f_files;
			vfs_stats_set_nb_files ( p, size);
			vfs_stats_set_blocks_size ( p, sts.f_bsize);
			size = sts.f_blocks;
			vfs_stats_set_blocks ( p, size);
			size = sts.f_bfree;
			vfs_stats_set_blocks_free ( p, size);
				/*vfs_stats_set_id ( p, sts.f_fsid);*/

				if ( !gw_device_manager_disk_is_mounted_from_vfs_stats ( p) && !vfs_stats_get_ismounted ( p))
				{
						gw_device_manager_disk_umount_from_vfs_stats ( p);
				}


			}
			else
			{
				/* The virtual file system cannot be mounted */
				vfs_stats_set_ismounted ( p, FALSE);
			}

#ifdef GW_DEBUG_MODE
			vfs_stats_display ( p);
#endif

			return 0;
		}
	}

	if ( partition != NULL )
	{
		g_free ( partition);
		partition = NULL;
	}

	endmntent ( tab);
#endif
	return -1;
}


gboolean gw_device_manager_disk_mount_from_vfs_stats ( struct vfs_stats *p)
{
#if defined ( OS_LINUX) || defined ( OS_CYGWIN) || defined ( OS_FREEBSD) || defined ( OS_OPENBSD) || defined ( OS_NETBSD)
	/* Mount device */
	int pid;		/* pid number		*/
	int status;	/* process status		*/
	int fd[2];		/* pipe descriptor		*/
	char *argv[4] = {"mount", vfs_stats_get_dir ( p), 0};
 	char retour[201];


#ifdef GW_DEBUG_MODE
	gw_application_manager_log_msg ( 0, __FILE__, __LINE__, __PRETTY_FUNCTION__, NULL);
#endif

	/* Creates pipe */
	if ( pipe ( fd) != 0 )
	{
		/* Cannot create pipe */

		perror ( "pipe");

		return FALSE;
	}

	/* Creates a fork */
	pid = fork (  );

	if ( pid == 0 )
	{
		/* If it's the child */
		close ( fd[0]);
		close ( 2);
		close ( 1);

		/* Redirect the output error to the pipe */
		dup2 ( fd[1], 2);
		close ( fd[1]);

//		if ( execv( "/bin/mount", argv) == -1 )
		if ( execv( GW_PROG_PATH_MOUNT, argv) == -1 )
		{
			perror ( "execv");
		}

		exit ( 127);
	}
	else
	{
		/* If it's the parent */
		if ( pid == -1 )
		{
			perror ( "fork");
		}
	}

	close ( fd[1]);

        	/* Gets the pipe data */
	gw_pipe_get_data ( fd[0], retour);

	if ( waitpid ( pid, & status, 0) == -1 )
	{
		perror ( "waitpid");
	}

	 if ( WEXITSTATUS ( status) != 0 )
	 {
		 return FALSE;
	 }
#endif
	 return TRUE;
}


gboolean gw_device_manager_disk_umount_from_vfs_stats( struct vfs_stats *p)
{
#if defined ( OS_LINUX) || defined ( OS_CYGWIN) || defined ( OS_FREEBSD) || defined ( OS_OPENBSD) || defined ( OS_NETBSD)
	/* Umout device */
	gint pid;		/* pid number		*/
	gint status;	/* process status		*/
	gint fd[2];		/* pipe descriptor		*/
	gchar *argv[4] =  {"umount", vfs_stats_get_dir ( p), 0};
 	gchar retour[201];


#ifdef GW_DEBUG_MODE
	gw_application_manager_log_msg ( 0, __FILE__, __LINE__, __PRETTY_FUNCTION__, NULL);
#endif

	/* Creates pipe */
	if ( pipe ( fd) != 0 )
	{
		perror ( "pipe");

		return FALSE;
	}

	/* Creates fork */
	pid = fork ( );
	if ( pid == 0 )
	{
		/* If it's the child */
		close ( fd[0]);
		close ( 2);
		close ( 1);

		/* Redirect the output error to the pipe */
		dup2 ( fd[1], 2);
		close ( fd[1]);

//		if ( execv ( "/bin/umount", argv) == -1 )
		if ( execv ( GW_PROG_PATH_UMOUNT, argv) == -1 )
		{
			perror ( "execv");
		}

		exit ( 127);
	}
	else
	{
		/* If it's the parent */
		if ( pid == -1 )
		{
			perror ( "fork");
		}
	}

	close ( fd[1]);

        	/* Gets the pipe data */
	gw_pipe_get_data ( fd[0], retour);

	/* Waits the child process end */
	if ( waitpid ( pid, & status, 0) == -1 )
	{
		perror ( "waitpid");
	}

	 if ( WEXITSTATUS ( status) != 0 )
	 {
//		 gw_msg_box_create ( "Mount device", retour);

		 return FALSE;
	 }

#endif
	return TRUE;
}

gint gw_pipe_get_data ( gint entry, gchar * line)
{
	gint nb;


#ifdef GW_DEBUG_MODE
	gw_application_manager_log_msg ( 0, __FILE__, __LINE__, __PRETTY_FUNCTION__, NULL);
#endif

	while ( (nb = read ( entry, line, 200)) > 0 )
	{
		line[nb] = '\0';
	};

	if ( nb < 0 )
	{
		perror ( "read");
	};

 	return 0;
}


gint gw_device_manager_disk_get_device_name ( const gchar *mount_point, gchar **device_name)
{
	gint result = -1;


#ifdef GW_DEBUG_MODE
	gw_application_manager_log_msg ( 0, __FILE__, __LINE__, __PRETTY_FUNCTION__, NULL);
#endif

	if ( mount_point != NULL )
	{
		if ( strlen ( mount_point) > 10 )
		{
			switch ( mount_point[10])
			{
				case 'a' : 	*device_name = g_strdup ( "/dev/fd0");
						result = 0;

						break;

				case 'b' : 	*device_name = g_strdup ( "/dev/fd1");
						result = 0;

						break;

				case 'c' : 	*device_name = g_strdup ( "/dev/sda1");
						result = 0;

						break;

				case 'd' : 	*device_name = g_strdup ( "/dev/sda2");
						result = 0;

						break;

				default :	break;

			}
		}
	}

	return result;
}


gboolean gw_device_manager_disk_is_mounted_from_vfs_stats ( struct vfs_stats *p)
{
	gboolean success = FALSE;
#if defined ( OS_LINUX) || defined ( OS_FREEBSD) || defined ( OS_OPENBSD) || defined ( OS_NETBSD)
	gchar *type = NULL;
#elif defined ( OS_WIN32) || defined ( OS_CYGWIN)
	HANDLE hdl_device = NULL;
	DWORD lp_bytes_returned;
	gchar *device_name = NULL;
#endif


#ifdef GW_DEBUG_DATA_COMPONENT
	gw_application_manager_log_msg ( 0, __FILE__, __LINE__, __PRETTY_FUNCTION__, NULL);
#endif

	if ( p != NULL )
	{
#if defined ( OS_LINUX) || defined ( OS_FREEBSD) || defined ( OS_OPENBSD) || defined ( OS_NETBSD)
		type = vfs_stats_get_type ( p);

		if ( (type != NULL) && (strncmp ( type, "supermount", 10) == 0) )
		{
			/* If the device is in supermount mode, try to acces it */

			success = gw_device_manager_disk_try_mount_from_vfs_stats ( p);
		}
		else
		{
			/* Returns the mount state */

			success = vfs_stats_get_ismounted ( p);
		}
#elif defined ( OS_WIN32) || defined ( OS_CYGWIN)
		/* I don't know with but the drive letter has to be passed without a trailing slash (ie "\\.\C:" instead of "\\.\C:\"). */
		device_name = g_strdup ( vfs_stats_get_name ( p));
		if ( device_name[strlen(device_name)-1] == '\\')
		{
			device_name[strlen(device_name)-1] = '\0';
		}

		if ( (hdl_device = CreateFile ( device_name, GENERIC_READ, FILE_SHARE_READ | FILE_SHARE_WRITE, NULL, OPEN_EXISTING, 0, 0)) != NULL )
		{
			success = DeviceIoControl ( hdl_device, IOCTL_STORAGE_CHECK_VERIFY, NULL, 0, NULL, 0, &lp_bytes_returned, NULL);
			CloseHandle ( hdl_device);
		}

		g_free ( device_name);
#endif
	}

	return success;
}


gboolean gw_device_manager_disk_can_mounted_from_vfs_stats ( struct vfs_stats *p)
{
#ifdef GW_DEBUG_DATA_COMPONENT
	gw_application_manager_log_msg ( 0, __FILE__, __LINE__, __PRETTY_FUNCTION__, NULL);
#endif

	if ( p != NULL )
	{
		if ( vfs_stats_get_ismounted ( p) == TRUE)
		{
			return TRUE;
		}
		else
		{
			return gw_device_manager_disk_try_mount_from_vfs_stats ( p);
		}
	}

	return FALSE;
}
